"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _promise = _interopRequireDefault(require("@babel/runtime/core-js/promise"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _rimraf = require("rimraf");

var _fs = require("fs");

var _path = require("path");

var _umiPlugin = require("umi-plugin");

var _getUserConfig = _interopRequireWildcard(require("af-webpack/getUserConfig"));

var _chalk = _interopRequireDefault(require("chalk"));

var _getPaths = _interopRequireDefault(require("./getPaths"));

var _getRouteConfig = _interopRequireDefault(require("./getRouteConfig"));

var _registerBabel = require("./registerBabel");

var _getConfig = require("./getConfig");

var _watch = require("./getConfig/watch");

var _getPlugins = _interopRequireDefault(require("./getPlugins"));

var _getWebpackConfig = _interopRequireDefault(require("./getWebpackConfig"));

var _chunksToMap = _interopRequireDefault(require("./utils/chunksToMap"));

var _send = _interopRequireWildcard(require("./send"));

var _FilesGenerator = _interopRequireDefault(require("./FilesGenerator"));

var _HtmlGenerator = _interopRequireDefault(require("./HtmlGenerator"));

var _createRouteMiddleware = _interopRequireDefault(require("./createRouteMiddleware"));

var debug = require('debug')('umi-build-dev:Service');

var Service =
/*#__PURE__*/
function () {
  function Service(cwd, _ref) {
    var pluginFiles = _ref.pluginFiles,
        babel = _ref.babel,
        entryJSTpl = _ref.entryJSTpl,
        routerTpl = _ref.routerTpl,
        hash = _ref.hash,
        preact = _ref.preact,
        extraResolveModules = _ref.extraResolveModules,
        _ref$libraryName = _ref.libraryName,
        libraryName = _ref$libraryName === void 0 ? 'umi' : _ref$libraryName,
        _ref$staticDirectory = _ref.staticDirectory,
        staticDirectory = _ref$staticDirectory === void 0 ? 'static' : _ref$staticDirectory,
        _ref$tmpDirectory = _ref.tmpDirectory,
        tmpDirectory = _ref$tmpDirectory === void 0 ? '.umi' : _ref$tmpDirectory,
        _ref$outputPath = _ref.outputPath,
        outputPath = _ref$outputPath === void 0 ? './dist' : _ref$outputPath;
    (0, _classCallCheck2.default)(this, Service);
    this.cwd = cwd || process.cwd();
    this.pluginFiles = pluginFiles;
    this.babel = babel;
    this.entryJSTpl = entryJSTpl;
    this.routerTpl = routerTpl;
    this.hash = hash;
    this.preact = preact;
    this.extraResolveModules = extraResolveModules;
    this.libraryName = libraryName;
    this.staticDirectory = staticDirectory;
    this.tmpDirectory = tmpDirectory;
    this.outputPath = outputPath;
    this.paths = (0, _getPaths.default)(this);
    this.registerBabel();
  }

  (0, _createClass2.default)(Service, [{
    key: "registerBabel",
    value: function registerBabel() {
      (0, _registerBabel.registerBabelForConfig)(this.babel, {
        paths: this.paths
      });
    }
  }, {
    key: "setRoutes",
    value: function setRoutes(routes) {
      this.routes = routes;
    }
  }, {
    key: "getWebpackRCConfig",
    value: function getWebpackRCConfig() {
      return (0, _getUserConfig.default)({
        cwd: this.cwd,
        disabledConfigs: ['entry', 'outputPath', 'hash']
      });
    }
  }, {
    key: "dev",
    value: function dev() {
      var _this = this;

      // 获取 .webpackrc 配置
      var returnedWatchWebpackRCConfig = null;

      try {
        var configObj = this.getWebpackRCConfig();
        this.webpackRCConfig = configObj.config;
        returnedWatchWebpackRCConfig = configObj.watch;
      } catch (e) {
        console.error(_chalk.default.red(e.message));
        debug('Get .webpackrc config failed, watch config and reload'); // 监听配置项变更，然后重新执行 dev 逻辑

        (0, _getUserConfig.watchConfigs)().on('all', function (event, path) {
          debug("[".concat(event, "] ").concat(path, ", unwatch and reload")); // 重新执行 dev 逻辑

          (0, _getUserConfig.unwatchConfigs)();

          _this.dev();
        });
        return;
      } // 获取用户 config.js 配置


      var returnedWatchConfig = null;

      try {
        var _configObj = (0, _getConfig.getConfig)(this.cwd, {
          force: true
        });

        this.config = _configObj.config;
        returnedWatchConfig = _configObj.watch;
      } catch (e) {
        console.error(_chalk.default.red(e.message));
        debug('Get config failed, watch config and reload'); // 监听配置项变更，然后重新执行 dev 逻辑

        (0, _getConfig.watchConfigs)().on('all', function (event, path) {
          debug("[".concat(event, "] ").concat(path, ", unwatch and reload")); // 重新执行 dev 逻辑

          (0, _watch.unwatch)();

          _this.dev();
        });
        return;
      }

      this.initRoutes();
      this.initPlugins(); // 生成入口文件

      var filesGenerator = new _FilesGenerator.default(this);

      try {
        filesGenerator.generate({
          onChange: function onChange() {
            _this.sendPageList();
          }
        });
      } catch (e) {
        console.error(_chalk.default.red(e.message));
        console.error(_chalk.default.red(e.stack));
        debug('Generate entry failed, watch pages and reload');
        filesGenerator.watch({
          onChange: function onChange() {
            filesGenerator.unwatch();

            _this.dev();
          }
        });
        return;
      }

      var webpackConfig = (0, _getWebpackConfig.default)(this);
      this.webpackConfig = webpackConfig;
      var isCompileDone = false;

      var _onCompileDone = function onCompileDone() {
        isCompileDone = true;
      }; // webpackConfig.output.publicPath = '/static/';


      require('af-webpack/dev').default({
        webpackConfig: webpackConfig,
        extraMiddlewares: [(0, _createRouteMiddleware.default)(this, {
          rebuildEntry: function rebuildEntry() {
            // return;
            if (!isCompileDone) {
              // 改写
              var defaultOnCompileDone = _onCompileDone;

              _onCompileDone = function onCompileDone() {
                debug('new compile done');
                filesGenerator.rebuild();
                defaultOnCompileDone(); // 再次改写

                _onCompileDone = function onCompileDone() {};
              };
            } else {
              filesGenerator.rebuild();
            }
          }
        })],
        afterServer: function afterServer(devServer) {
          _this.devServer = devServer;
          returnedWatchWebpackRCConfig(devServer, {
            beforeChange: function beforeChange() {
              _this.registerBabel();
            }
          });
          returnedWatchConfig(devServer);
          filesGenerator.watch();
        },
        onCompileDone: function onCompileDone() {
          _onCompileDone();
        },
        proxy: this.webpackRCConfig.proxy || {},
        // 支付宝 IDE 里不自动打开浏览器
        openBrowser: !process.env.ALIPAY_EDITOR
      });
    }
  }, {
    key: "initRoutes",
    value: function initRoutes() {
      this.routes = (0, _getRouteConfig.default)(this.paths, this.config);
    }
  }, {
    key: "initPlugins",
    value: function initPlugins() {
      this.plugins = (0, _getPlugins.default)({
        configPlugins: this.config.plugins,
        pluginsFromOpts: this.pluginFiles,
        cwd: this.cwd,
        babel: this.babel
      });
    }
  }, {
    key: "sendPageList",
    value: function sendPageList() {
      var pageList = this.routes.map(function (route) {
        return {
          path: route.path
        };
      });
      (0, _send.default)({
        type: _send.PAGE_LIST,
        payload: pageList
      });
    }
  }, {
    key: "build",
    value: function build() {
      var _this2 = this;

      this.webpackRCConfig = this.getWebpackRCConfig().config;
      this.config = (0, _getConfig.getConfig)(this.cwd).config;
      this.initRoutes();
      this.initPlugins();
      debug("Clean tmp dir ".concat(this.paths.tmpDirPath));
      (0, _rimraf.sync)(this.paths.absTmpDirPath);
      debug("Clean output path ".concat(this.paths.outputPath));
      (0, _rimraf.sync)(this.paths.absOutputPath);
      debug('Generate entry');
      var filesGenerator = new _FilesGenerator.default(this);
      filesGenerator.generate();
      var webpackConfig = (0, _getWebpackConfig.default)(this);
      this.webpackConfig = webpackConfig;
      return new _promise.default(function (resolve) {
        require('af-webpack/build').default({
          webpackConfig: webpackConfig,
          success: function success(_ref2) {
            var stats = _ref2.stats;

            if (process.env.RM_TMPDIR === 'none') {
              debug("Clean tmp dir ".concat(_this2.paths.tmpDirPath));
              (0, _rimraf.sync)(_this2.paths.absTmpDirPath);
            }

            debug("Bundle html files");
            var chunksMap = (0, _chunksToMap.default)(stats.compilation.chunks);

            try {
              var hg = new _HtmlGenerator.default(_this2, {
                chunksMap: chunksMap
              });
              hg.generate();
            } catch (e) {
              console.log(e);
            }

            debug('Move service-worker.js');
            var sourceSW = (0, _path.join)(_this2.paths.absOutputPath, _this2.staticDirectory, 'service-worker.js');
            var targetSW = (0, _path.join)(_this2.paths.absOutputPath, 'service-worker.js');

            if ((0, _fs.existsSync)(sourceSW)) {
              (0, _fs.renameSync)(sourceSW, targetSW);
            } // Usage:
            // - umi-plugin-yunfengdie


            (0, _umiPlugin.applyPlugins)(_this2.plugins, 'buildSuccess', null, {
              service: _this2
            });
            (0, _send.default)({
              type: _send.BUILD_DONE
            });
            resolve();
          }
        });
      });
    }
  }]);
  return Service;
}();

exports.default = Service;