"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfig = getConfig;
exports.watchConfigs = watchConfigs;

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _keys = _interopRequireDefault(require("@babel/runtime/core-js/object/keys"));

var _fs = require("fs");

var _path = require("path");

var _reactDevUtils = require("af-webpack/react-dev-utils");

var _chalk = _interopRequireDefault(require("chalk"));

var _lodash = _interopRequireDefault(require("lodash.isequal"));

var _didyoumean = _interopRequireDefault(require("didyoumean"));

var _requireindex = _interopRequireDefault(require("requireindex"));

var _constants = require("../constants");

var _watch2 = require("./watch");

var _createRouteMiddleware = require("../createRouteMiddleware");

var pluginsMap = (0, _requireindex.default)((0, _path.join)(__dirname, './configPlugins'));
var plugins = (0, _keys.default)(pluginsMap).map(function (key) {
  return pluginsMap[key].default();
});
var devServer = null;

function printError(messages) {
  if (devServer) {
    devServer.sockWrite(devServer.sockets, 'errors', typeof messages === 'string' ? [messages] : messages);
  }
}

function printWarn(messages) {
  if (devServer) {
    devServer.sockWrite(devServer.sockets, 'warns', typeof messages === 'string' ? [messages] : messages);
  }
}

function reload() {
  if (devServer) {
    devServer.sockWrite(devServer.sockets, 'content-changed');
  }
}

function restart(why) {
  if (devServer) {
    (0, _reactDevUtils.clearConsole)();
    console.log(_chalk.default.green("Since ".concat(why, ", try to restart server")));
    (0, _watch2.unwatch)();
    devServer.close();
    process.send({
      type: 'RESTART'
    });
  }
}

function getConfig(cwd) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var files = _constants.CONFIG_FILES.filter(function (file) {
    return (0, _fs.existsSync)((0, _path.join)(cwd, file));
  }).map(function (file) {
    return (0, _path.join)(cwd, file);
  }); // 从两个地方读取配置，但只允许有一个


  if (files.length === 2) {
    printWarn("\n\u4F60\u6709\u4E24\u4E2A\u914D\u7F6E\u6587\u4EF6\uFF0C".concat(files[0], " \u548C ").concat(files[1], "\uFF0C\u8BF7\u5220\u9664\u5176\u4E2D\u4E00\u4E2A\uFF0C\u6211\u4EEC\u4F1A\u4F7F\u7528\u5176\u4E2D\u7684 ").concat(files[0], "\u3002\n    ").trim());
  }

  var file = null;
  var config = {}; // 执行插件方法时的上下文

  var context = {
    cwd: cwd,
    file: file,
    printError: printError,
    printWarn: printWarn,
    restart: restart,
    reload: reload
  };

  if (files.length) {
    file = files[0];
    var relativeFile = file.replace("".concat(cwd, "/"), '');
    context.relativeFile = relativeFile; // 强制读取，不走 require 缓存

    if (opts.force) {
      _constants.CONFIG_FILES.forEach(function (configFile) {
        delete require.cache[(0, _path.join)(cwd, configFile)];
      });
    }

    try {
      config = require(file); // eslint-disable-line
    } catch (e) {
      var msg = "\u914D\u7F6E\u6587\u4EF6 \"".concat(relativeFile, "\" \u89E3\u6790\u51FA\u9519\uFF0C\u8BF7\u68C0\u67E5\u8BED\u6CD5\u3002\n\r\n").concat(e.toString());
      printError(msg);
      throw new Error(msg);
    }

    if (config.default) {
      config = config.default;
    } // 把外层 context 的内容复制进来，用内层的覆盖它


    if (config.context && config.pages) {
      (0, _keys.default)(config.pages).forEach(function (key) {
        var page = config.pages[key];
        page.context = (0, _extends2.default)({}, config.context, page.context);
      });
    } // pages 配置补丁
    // /index -> /index.html
    // index -> /index.html


    if (config.pages) {
      var htmlSuffix = !!(config.exportStatic && typeof config.exportStatic === 'object' && config.exportStatic.htmlSuffix);
      config.pages = (0, _keys.default)(config.pages).reduce(function (memo, key) {
        var newKey = key; // Remove it if no break
        // if (newKey === '/') {
        //   newKey = '/index.html';
        // }

        if (htmlSuffix && newKey.slice(-5) !== '.html') {
          newKey = "".concat(newKey, ".html");
        }

        if (newKey.charAt(0) !== '/') {
          newKey = "/".concat(newKey);
        }

        memo[newKey] = config.pages[key];
        return memo;
      }, {});
    } // 校验


    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = (0, _getIterator2.default)(plugins), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _plugin = _step.value;
        var name = _plugin.name;

        if (config[name] && _plugin.validate) {
          try {
            _plugin.validate.call((0, _extends2.default)({}, context, {
              plugin: _plugin,
              name: name,
              watch: _watch2.watch.bind(null, name),
              unwatch: _watch2.unwatch.bind(null, name)
            }), config[name], config);
          } catch (e) {
            // 校验出错后要把值设到缓存的 config 里，确保 watch 判断时才能拿到正确的值
            if (opts.setConfig) {
              opts.setConfig(config);
            }

            printError(e.message);
            throw new Error("\u914D\u7F6E ".concat(name, " \u6821\u9A8C\u5931\u8D25, ").concat(e.message));
          }
        }
      } // 找下不匹配的 name

    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    var pluginNames = plugins.map(function (p) {
      return p.name;
    });
    (0, _keys.default)(config).forEach(function (key) {
      if (!pluginNames.includes(key)) {
        if (opts.setConfig) {
          opts.setConfig(config);
        }

        var affixmsg = "\u9009\u62E9 \"".concat(pluginNames.join(', '), "\" \u4E2D\u7684\u4E00\u9879\uFF0C\u8BE6\u89C1 https://fengdie.alipay-eco.com/doc/h5app/configuration");
        var guess = (0, _didyoumean.default)(key, pluginNames);
        var midMsg = guess ? "\u4F60\u662F\u4E0D\u662F\u60F3\u914D\u7F6E \"".concat(guess, "\" \uFF1F \u6216\u8005") : '请';

        var _msg = "\"".concat(relativeFile, "\" \u4E2D\u914D\u7F6E\u7684 \"").concat(key, "\" \u5E76\u975E\u7EA6\u5B9A\u7684\u914D\u7F6E\u9879\uFF0C").concat(midMsg).concat(affixmsg);

        printError(_msg);
        throw new Error(_msg);
      }
    });
  }

  var configFailed = false;
  return {
    file: file,
    config: config,
    watch: function watch(_devServer) {
      devServer = _devServer; // 配置插件的监听

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = (0, _getIterator2.default)(plugins), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var _plugin5 = _step2.value;
          var _name4 = _plugin5.name;

          if (_plugin5.watch) {
            _plugin5.watch.call((0, _extends2.default)({}, context, {
              plugin: _plugin5,
              name: _name4,
              watch: _watch2.watch.bind(null, _name4),
              unwatch: _watch2.unwatch.bind(null, _name4)
            }), config[_name4], config);
          }
        } // 配置文件的监听

      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      (0, _watch2.watch)('CONFIG_FILES', _constants.CONFIG_FILES).on('all', function (event, path) {
        console.log("[DEBUG] [".concat(event, "] ").concat(path));

        try {
          var _getConfig = getConfig(cwd, {
            force: true,
            setConfig: function setConfig(newConfig) {
              config = newConfig;
            }
          }),
              newConfig = _getConfig.config; // 更新 middleware 的配置


          (0, _createRouteMiddleware.setConfig)(newConfig); // 从失败中恢复过来，需要 reload 一次

          if (configFailed) {
            configFailed = false;
            reload();
          }

          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = (0, _getIterator2.default)(plugins), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              var _plugin4 = _step3.value;
              var _name3 = _plugin4.name,
                  onChange = _plugin4.onChange;

              if (!(0, _lodash.default)(newConfig[_name3], config[_name3])) {
                if (onChange) {
                  onChange.call((0, _extends2.default)({}, context, {
                    plugin: _plugin4,
                    name: _name3,
                    watch: _watch2.watch.bind(null, _name3),
                    unwatch: _watch2.unwatch.bind(null, _name3)
                  }), newConfig[_name3], config[_name3], newConfig);
                }
              }
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
                _iterator3.return();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }

          config = newConfig;
        } catch (e) {
          configFailed = true;
          console.error(_chalk.default.red("watch handler failed, since ".concat(e.message)));
          console.error(e);
        }
      });
    }
  };
}

function watchConfigs() {
  return (0, _watch2.watch)('CONFIG_FILES', _constants.CONFIG_FILES);
}