"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _getIterator2 = _interopRequireDefault(require("@babel/runtime/core-js/get-iterator"));

var _fs = require("fs");

var _path = require("path");

var _assert = _interopRequireDefault(require("assert"));

var _constants = require("./constants");

var _winPath = _interopRequireDefault(require("./winPath"));

var DOT_JS = '.js';
var EXT_NAMES = ['.js', '.jsx', '.ts', '.tsx'];

function _default(paths) {
  var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var routeConfigFile = routesConfigExists(paths.cwd);
  var routes = routeConfigFile ? getRoutesByConfig(routeConfigFile) : getRoutesByPagesDir(paths);

  if (config.exportStatic) {
    routes.forEach(function (route) {
      if (route.path.indexOf(':') > -1) {
        throw new Error("Variable path ".concat(route.path, " don't work with exportStatic"));
      }

      if (typeof config.exportStatic === 'object' && config.exportStatic.htmlSuffix) {
        route.path = addHtmlSuffix(route.path);
      }
    });
  }

  return routes;
}

function routesConfigExists(root) {
  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator2.default)(_constants.ROUTES_CONFIG_FILE), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _routeConfig = _step.value;

      if ((0, _fs.existsSync)((0, _path.join)(root, _routeConfig))) {
        return (0, _path.join)(root, _routeConfig);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

function addHtmlSuffix(path) {
  return path.slice(-1) === '/' ? path : "".concat(path, ".html");
}

function getRoutesByConfig(routesConfigFile) {
  var routesConfig = JSON.parse((0, _fs.readFileSync)(routesConfigFile));
  (0, _assert.default)(Array.isArray(routesConfig), "router config must be Array, but got ".concat(routesConfig));
  return routesConfig;
}

function variablePath(path) {
  return path.replace(/\$/g, ':');
}

function getRoutesByPagesDir(paths) {
  var dirPath = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';
  var cwd = paths.cwd,
      absPagesPath = paths.absPagesPath;
  var ret = [];
  var path = (0, _path.join)(absPagesPath, dirPath);

  if ((0, _fs.existsSync)(path)) {
    var files = (0, _fs.readdirSync)(path);
    files.forEach(function (file) {
      // 包含 ., .., 以及其他 dotfile
      if (file.charAt(0) === '.') return;
      var filePath = (0, _path.join)(path, file);
      var stats = (0, _fs.statSync)(filePath);
      var ext = (0, _path.extname)(file);

      if (stats.isFile() && EXT_NAMES.indexOf(ext) > -1) {
        var fullPath = (0, _path.join)(dirPath, (0, _path.basename)(file, ext));
        ret.push({
          path: (0, _winPath.default)("/".concat(variablePath(fullPath))).replace(/\/index$/, '/'),
          exact: true,
          component: "./".concat((0, _path.relative)(cwd, filePath))
        });
      } else if (stats.isDirectory()) {
        var routerFound = false;

        var _fullPath = (0, _path.join)(dirPath, file);

        var _iteratorNormalCompletion2 = true;
        var _didIteratorError2 = false;
        var _iteratorError2 = undefined;

        try {
          for (var _iterator2 = (0, _getIterator2.default)(_constants.ROUTE_FILES), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var _routeFile = _step2.value;

            if ((0, _fs.existsSync)((0, _path.join)(absPagesPath, _fullPath, _routeFile))) {
              if ((0, _fs.existsSync)((0, _path.join)(absPagesPath, "".concat(_fullPath).concat(DOT_JS)))) {
                throw new Error("\u8DEF\u7531\u51B2\u7A81\uFF0Csrc/page \u76EE\u5F55\u4E0B\u540C\u65F6\u5B58\u5728 \"".concat(_fullPath).concat(DOT_JS, "\" \u548C \"").concat((0, _path.join)(_fullPath, _routeFile), "\"\uFF0C\u4E24\u8005\u6307\u5411\u540C\u4E00\u8DEF\u7531\u3002"));
              }

              ret.push({
                path: (0, _winPath.default)("/".concat(variablePath(_fullPath))).replace(/\/index$/, '/'),
                exact: true,
                component: "./".concat((0, _path.relative)(cwd, (0, _path.join)(absPagesPath, _fullPath, _routeFile)))
              });
              routerFound = true;
              break;
            }
          }
        } catch (err) {
          _didIteratorError2 = true;
          _iteratorError2 = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
              _iterator2.return();
            }
          } finally {
            if (_didIteratorError2) {
              throw _iteratorError2;
            }
          }
        }

        if (!routerFound) {
          ret = ret.concat(getRoutesByPagesDir(paths, _fullPath));
        }
      }
    });
  }

  return ret;
}